#!/usr/bin/env python

"""
XSL-based form templating.
"""

import Constants
import libxslt

"""
import libxml2

def quiet(context, s):
    pass

libxml2.registerErrorHandler(quiet, None)
"""

def path_to_node(node, attribute_ref, name):

    """
    Generate an XSLForms path to the given 'node', producing an attribute
    reference if 'attribute_ref' is true; for example:

    /package#1/discriminators#5/discriminator#1/category

    Otherwise an element reference is produced; for example:

    /package#1/discriminators#5/discriminator#1

    Use the given 'name' to complete the path if an attribute reference is
    required (and if a genuine attribute is found at the context node -
    otherwise 'name' will be None and the context node will be treated like an
    attribute).
    """

    l = []
    # Skip attribute reference.
    if node.type == "attribute":
        node = node.parent
    # Manually insert the attribute name if defined.
    if attribute_ref:
        if name is not None:
            l.insert(0, name)
            l.insert(0, Constants.path_separator)
        else:
            l.insert(0, node.name)
            l.insert(0, Constants.path_separator)
            node = node.parent
    # Element references.
    while node is not None and node.type != "document_xml":
        l.insert(0, str(int(node.xpathEval("count(preceding-sibling::*) + 1"))))
        l.insert(0, Constants.pair_separator)
        l.insert(0, node.name)
        l.insert(0, Constants.path_separator)
        node = node.parent
    return "".join(l)

def path_to_context(context, attribute_ref):

    """
    As a libxslt extension function, return a string containing the XSLForms
    path to the 'context' node, using the special "this-name" variable to
    complete the path if an attribute reference is required (as indicated by
    'attribute_ref' being set to true).
    """

    pctxt = libxslt.xpathParserContext(_obj=context)
    context = pctxt.context()
    node = context.contextNode()
    transform_context = context.transformContext()
    name_var = transform_context.variableLookup("this-name", None)
    if name_var is not None:
        name = name_var[0].content
    else:
        name = None
    return path_to_node(node, attribute_ref, name)

def this_position(context):
    return path_to_context(context, 0)

def field_name(context):
    return path_to_context(context, 1)

libxslt.registerExtModuleFunction("this-position", "http://www.boddie.org.uk/ns/xmltools/template", this_position)
libxslt.registerExtModuleFunction("field-name", "http://www.boddie.org.uk/ns/xmltools/template", field_name)

# vim: tabstop=4 expandtab shiftwidth=4
